/*******************************ADC************************************/

void adc_init(void) 
{
    //ADC Setup
    ADMUX = (1<<REFS0) | (1<<MUX1);
    ADCSRA = (1<<ADEN) | (1<<ADPS1) | (1<<ADPS2);
    
}

/*******************************BASIC*********************************/

void basic_init(void)
{
   // turn boost off
    TCCR0B = 0;
    BOOST_DDR |= _BV(BOOST);
    BOOST_PORT &= ~_BV(BOOST); // pull boost fet low

    wdt_disable();
    wdt_enable(WDTO_2S);
    kickthedog();

    // set up analog comparator
    ACSR = _BV(ACBG) | _BV(ACIE); // use bandgap, intr. on toggle!
    _delay_ms(1);
    // settle!
    if (ACSR & _BV(ACO)) 
        {
        // hmm we should not interrupt here
        ACSR |= _BV(ACI);

        // even in low power mode, we run the clock
        clock_init();  

        } 
    else 
        {
    	// we aren't in low power mode so init stuff

        // set up button interrupts
    	button_init();
    
    	VFDSWITCH_PORT &= ~_BV(VFDSWITCH);
    
    	// set up interrupt for DCF receiver
    	EICRA = _BV(ISC00);
    	EIMSK = _BV(INT0);
  
    	displaymode = NONE;
    	vfd_init();
    	boost_init();
    	sei();

    	time.r = eeprom_read_byte((uint8_t *)EE_REGION);
	effect_ctrl.effect = eeprom_read_byte((uint8_t *)EE_EFFECT);    
	effect_ctrl.light = eeprom_read_byte((uint8_t *)EE_LIGHT);
        tempoffset = eeprom_read_byte((uint8_t *)EE_OFFSET);
        if (tempoffset > 30)
            tempoffset = 15;
    	clock_init();  
        adc_init();
        read_temperature();
        
        pwm_blue = PWM_MIN;
        pwm_green = PWM_MIN;
        pwm_red = PWM_MIN;
        if (effect_ctrl.light == FALSE)
            recreate_pwm_sequence();

    	speaker_init();

    	beep(5000, 1);
        beep(8000, 1);
	beep(4000, 1);
	
	displaymode = NONE;

    	display_str("tobi uhr", TRUE);

    	displaymode = SHOW_TIME;
  	}
}

/*******************************BOOST*********************************/

// We control the boost converter by changing the PWM output
// pins
void boost_init(void)
    {
    ctrl_struct.brightness = eeprom_read_byte((uint8_t *)EE_BRIGHT);
    // Set PWM value, don't set it so high that
    // we could damage the MAX chip or display
    if (ctrl_struct.brightness > 100)
        ctrl_struct.brightness = 100;

    // Or so low its not visible
    if (ctrl_struct.brightness < 10)
        ctrl_struct.brightness = 10;

    setboost (ctrl_struct.brightness);

    // fast PWM, set OC0A (boost output pin) on match
    TCCR0A = _BV(WGM00) | _BV(WGM01);  

    // Use the fastest clock
    TCCR0B = _BV(CS00);
 
    TCCR0A |= _BV(COM0A1);
    TIMSK0 |= _BV(TOIE0); // turn on the interrupt for muxing
    sei();
} 

/*******************************Buttons********************************/

void button_init(void) 
{
    DDRB = _BV(VFDCLK) | _BV(VFDDATA) | _BV(SPK1) | _BV(SPK2);
    DDRD = _BV(BOOST) | _BV(VFDSWITCH) | _BV(LED3);
    DDRC = _BV(VFDLOAD) | _BV(VFDBLANK) | _BV(LED1) | _BV(LED2) | _BV(LED4);
    PORTD = _BV(BUTTON1) | _BV(BUTTON3) | _BV(DCF);
    PORTB = _BV(BUTTON2);

    //enable this and disable other line for original state
    PCICR = _BV(PCIE0) | _BV(PCIE2);
    //PCICR = _BV(PCIE0) | _BV(PCIE1) | _BV(PCIE2);

    PCMSK0 = _BV(PCINT0);

    //enable this and disable other line for original state
    PCMSK2 = _BV(PCINT21) | _BV(PCINT20);    
    //PCMSK2 = _BV(PCINT17) | _BV(PCINT21) | _BV(PCINT20);

 
}

/*****************************RTC & ALARM******************************/

void clock_init(void) 
{
    // we store the time in EEPROM when switching from power modes so its
    // reasonable to start with whats in memory
    time.h = eeprom_read_byte((uint8_t *)EE_HOUR) % 24;
    time.m = eeprom_read_byte((uint8_t *)EE_MIN) % 60;
    time.s = eeprom_read_byte((uint8_t *)EE_SEC) % 60;
    time.z = eeprom_read_byte((uint8_t *)EE_ZONE);

    // Set up the stored alarm time and date
    alarm_m = eeprom_read_byte((uint8_t *)EE_ALARM_MIN) % 60;
    alarm_h = eeprom_read_byte((uint8_t *)EE_ALARM_HOUR) % 24;

    time.y = eeprom_read_byte((uint8_t *)EE_YEAR) % 100;
    time.mo = eeprom_read_byte((uint8_t *)EE_MONTH) % 13;
    time.d = eeprom_read_byte((uint8_t *)EE_DAY) % 32;

    
    // we lost power at some point so lets alert the user
    // that the time may be wrong (the clock still works)
    ctrl_struct.timeunknown = TRUE;
  
    // we lost power at some point so reset the countdown
    countdown = 0;
    ctrl_struct.countdown_is_set = FALSE;
    ctrl_struct.countdown_alarming = FALSE;

    // have we read the time & date from eeprom?
    ctrl_struct.restored = TRUE;
    ctrl_struct.dcf_sync = FALSE;
    ctrl_struct.dst = eeprom_read_byte((uint8_t *)EE_DST);
    ctrl_struct.alarm_on = eeprom_read_byte((uint8_t *)EE_ALARM_STATE);
  
    //reset dcf_date, setting check variables makes sync 1 min faster if time is still correct.
    dcf_struct.week_day = 0;
    dcf_struct.day = 0;
    dcf_struct.month = 0;
    dcf_struct.year = 0;
    dcf_struct.day_check = time.d;
    dcf_struct.month_check = time.mo;
    dcf_struct.year_check = time.y;
    dcf_struct.date_parity = 0;

    //reset dcf_time, setting check variables makes sync 1 min faster if time is still correct.
    dcf_struct.minute = 0;
    dcf_struct.minute_parity = 0;
    dcf_struct.hour = 0;
    dcf_struct.minute_check = time.m;
    dcf_struct.hour_check = time.h;
    dcf_struct.hour_parity = 0;
    dcf_struct.timezone = 0;

    // set alarm timer
    alarmtimer = MAXALARMTIMER;
    
    // Turn on the RTC by selecting the external 32khz crystal
    // 32.768 / 32 = (256*4) which is exactly an 8-bit timer overflow
    ASSR |= _BV(AS2); // use crystal
    TCCR2B = _BV(CS21) | _BV(CS20); // div by 32
    // We will overflow four times a second, and call an interrupt

    // enable interrupt
    TIMSK2 = _BV(TOIE2);
  
    // enable all interrupts!
    sei();
}

/*******************************SPEAKER********************************/
// Set up the speaker to prepare for beeping!

void speaker_init(void) 
{
    // read the preferences for high/low volume
    ctrl_struct.volume = eeprom_read_byte((uint8_t *)EE_VOLUME);

    // We use the built-in fast PWM, 8 bit timer
    PORTB |= _BV(SPK1) | _BV(SPK2); 

    // Turn on PWM outputs for both pins
    TCCR1A = _BV(COM1B1) | _BV(COM1B0) | _BV(WGM11);
    if (ctrl_struct.volume) 
        TCCR1A |= _BV(COM1A1);
    TCCR1B = _BV(WGM13) | _BV(WGM12);

    // start at 4khz:  250 * 8 multiplier * 4000 = 8mhz
    ICR1 = 250;
    OCR1B = OCR1A = ICR1 / 2;
}

/****************************SPI(DISPLAY)******************************/

// Setup SPI

void vfd_init(void) 
{
    SPCR  = _BV(SPE) | _BV(MSTR) | _BV(SPR0);
}
